// C Source File
// Created 1/1/2006; 10:06:20 AM
// Tron v1.00 - By Tyler Cassidy
// (C) Copyright 2006 Tyler Cassidy. All Rights Reserved.
// Email: altadogsledder89@yahoo.ca
//        tyler@tylerc.org
// Website: none yet
// ** Note: I will not be responsible for anything that this game game does to your calculator!

// Definitions
// #define MIN_AMS 200 // Only Enable this if you are not using TIGCC IDE

// Header Files
#include <tigcclib.h> // All TIGCC Header Files
#include "extgraph.h" // ExtGraph Library v2.00 B5
#include "tron.h" // Tron Header File
#include "ai.h" // Artificial Intellegence Header File
#include "engine.h" // Game Engine Header File

/// Game Engine
// Slow's Down Game Function - void slow_game(short loop)
void slow_game(short loop) {
	short i;
	for(i=0; i<loop; i++) randomize(); // Slow Down
	// End of Slow's Down Game Function
}

// Clear Patch - void clear_patch(short x_left, short x_width, short y_top, short y_height)
void clear_patch(short x_left, short x_width, short y_top, short y_height) {
	short i;
	for(i=y_top; i<=y_top + y_height; i++) {
		GrayDrawLine(x_left, i, x_left + x_width, i, 0); // Draw White Line
	}
	// End of Clear Patch
}

// Play Tron Function - void TronEngine_PlayTron(void)
void TronEngine_PlayTron(void) {
	// Do we need to setup?
	if (game_on == 0) {
    // Why yes we do :-)
		// Clear Screen
		GrayClearScreen();
		
		/// Setup some variable stuff
		game_score = 0; // Set score to 0
		game_over = 0; // Game is not over!
		game_paused = 0; // Game is not paused.

	  gameplay.your_dir = 1; // 1 = right
		gameplay.their_dir = 3; // 3 = left
		gameplay.comp_last_hum_dir = 1; // 1 = right
		gameplay.your_x = 3; // You will be set on 3x3
		gameplay.your_y = 3;
		gameplay.their_x = LCD_WIDTH - 3; // Comp will be set on 156x96 on 89 and 236x124 on 92+/v200
		gameplay.their_y = LCD_HEIGHT - 3;
			
		// Lets now Draw the Walls
		GrayDrawLine(0, 0, LCD_WIDTH - 1, 0, 3); // Black Line on Top
		GrayDrawLine(0, 0, 0, LCD_HEIGHT - 1, 3); // Black Line on Left
		GrayDrawLine(0, LCD_HEIGHT - 1, LCD_WIDTH - 1, LCD_HEIGHT - 1, 3); // Black Line on Bottom
		GrayDrawLine(LCD_WIDTH - 1, 0, LCD_WIDTH - 1, LCD_HEIGHT - 1, 3); // Black Line on Right
   
    // Turn Game On
		game_on = 1; // Play Tron!!!
	}
	
	/// Lets Play Tron!
  // Initialize AI
	AI_Initialize(); // Set New Direction, etc.

	// Lets setup speed timing
	speed_timing = 0;
	
	/// Play Game Loop
	while(1) {
    // Keyboard Detection
		if (!OSdequeue(&key, kbq)) {
			// Arrow Key Detection
			if (key == KEY_LEFT) gameplay.your_dir = 3; // 3 = left
			if (key == KEY_UP) gameplay.your_dir = 0; // 0 = up
			if (key == KEY_RIGHT) gameplay.your_dir = 1; // 1 = right
			if (key == KEY_DOWN) gameplay.your_dir = 2; // 2 = down
			
			// Esc Key
			if (key == KEY_ESC) show_pausemenu();
			
			// F5 Key
			if (key == KEY_F5) {
				game_on = 0; // Turn game off
				game_sequence = QUIT_GAME; // Goto Quit Game Sequence
				break; // Break out of Loop
			}
		}
    
    /// Speed Control Here
    if (speed_timing >= ((5 - game_speed) * 200)) {
      // Lets Draw the Pointers
  		EN_DrawPointer(); // Draw Pointer
	  	EN_CollideDetect(); // Check for Collision
		  game_score++; // Increment Game Score
    
      // AI Updates/Recalculation Functions	
      gameplay.comp_dist--; // Decrement Computer Distance
    	if (gameplay.your_dir != gameplay.comp_last_hum_dir) AI_RecalculateDirDist(); // Possible Cutoff Detection Function
      if (gameplay.comp_dist <= 1) AI_ChangeDir(); // Change Direction and Distance

      /// Game Over
      // 1 = you, 2 = them, 3 = tie, 0 = no game over, 4 = abort
      if (game_over > 0 && game_over <= 3) {
      	game_on = 0; // Game is off now
      	// Did you make it into the High Scores?
      	if (game_over == 2 && HighscoreQualify() == TRUE) {
      		HighscoreAddScore();
      		show_gameover();
      		game_sequence = HIGHSCORES_MENU;
      	}
      	
      	// No Highscore Entry, than just show Gameover with the score
      	else show_gameover();
      	
        // Lets bust this joint!
      	break; // Break out of this loop
      }
      // Abort, Game Over
      if (game_over == 4) break; // Abort Loop!
      /// End of Game Over
      
      // Lets now reset the speed_timing
      speed_timing = 0;
    }
    speed_timing++; // Increment Speed Timing
    /// End of Speed Timing
  }
	// End of Play Tron Function
}

// Draw Pointer Function - void EN_DrawPointer(void)
void EN_DrawPointer(void) {
	// Lets Draw the Pointers First
	GrayDrawLine(gameplay.your_x, gameplay.your_y, gameplay.your_x, gameplay.your_y, 3); // Nice Black Dot for You
	GrayDrawLine(gameplay.their_x, gameplay.their_y, gameplay.their_x, gameplay.their_y, 3); // Nice Black Dot for Them
	
	// Update your_x, etc.
	if (gameplay.your_dir == 0) gameplay.your_y--; // Decrement from Y
	if (gameplay.your_dir == 1) gameplay.your_x++; // Increment to X
	if (gameplay.your_dir == 2) gameplay.your_y++; // Increment to Y
	if (gameplay.your_dir == 3) gameplay.your_x--; // Decrement from X
	
	// Update their_x, etc.
	if (gameplay.their_dir == 0) gameplay.their_y--; // Decrement from Y
	if (gameplay.their_dir == 1) gameplay.their_x++; // Increment to X
	if (gameplay.their_dir == 2) gameplay.their_y++; // Increment to Y
	if (gameplay.their_dir == 3) gameplay.their_x--; // Decrement from X
	
	// End of Draw Pointer Function
}

// Collision Detection - void EN_CollideDetect(void)
void EN_CollideDetect(void) {
	// Good thing I can use Get Pix because these are Black Lines, meaning both planes are filled
	// First you
	if (GetPix(gameplay.your_x, gameplay.your_y) == TRUE) {
		// You Hit something
		game_over = 1; // Game is so over man! You lose!; 1 = you, 2 = them, 3 = tie, 0 = no game over, 4 = abort
	}
	
	// Then The Opponent
	if (GetPix(gameplay.their_x, gameplay.their_y) == TRUE) {
		// Opponent Hit Something
		if (game_over == 1) game_over = 3; // You both died! In a tie.
		else game_over = 2; // Game over for opponent!
	}
	
	// End of Collision Detection
}

// Clear Highscores - void ClearHighScores(void)
void ClearHighScores(void) {
	short i;
	
	for(i=1; i<=10; i++) {
		gamehighscores.score[i - 1] = 0;
	}
	gamehighscores.num_scores = 10;
	
	// End of Clear Highscores
}

// Highscore Qualifier - short HighscoreQualify(void)
short HighscoreQualify(void) {
  short i;

  // What is the score?
  if (game_score <= 0) return FALSE; // Dont want a bunch of 0 Scores :), so return FALSE
 
	// Did we make it into the top ten?
  for(i=1; i<=10; i++) {
		// Where do we have to place?
		if (game_score > gamehighscores.score[i - 1]) return TRUE; // You made it!
		if (game_score == gamehighscores.score[i - 1]) return FALSE; // Same Score already added, return FALSE
	}
	
	// Test Failed, return TRUE, he
	return FALSE;
	
	// End of Highscore Qualifier
}

// Highscore Enter - void HighscoreAddScore(void)
void HighscoreAddScore(void) {
	short i, i_place;
	i_place = 0;
	
	// Lets see where we have to place the entry
	for(i=1; i<=10; i++) {
		// Lets find out the weakest link!
		if (game_score > gamehighscores.score[i - 1]) {
			i_place = i; // We got the Marker
			break; // Break
		}
	}
	
	// Do we have to move scores down?
	if (i_place < 10) {
		for(i=10; i>=i_place; i--) {
			gamehighscores.score[i - 1] = gamehighscores.score[i - 2]; // Move Score Down
		}
	}
	
	// Lets add the Score
	gamehighscores.score[i_place - 1] = game_score; // Score Added
	
	// End of Highscore Enter
}